/* xscreensaver-command, Copyright (c) 1991-2003 Jamie Zawinski <jwz@jwz.org>
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or 
 * implied warranty.
 */

/* Great; a "put me in your docs" license.  That means we have to credit jwz
 * for writing xscreensaver--so we can use his code to disable it.  Ironic? */

#include "global.h"
#include "DisableXScreenSaver.h"

#include <stdio.h>
#include <stdlib.h>

#include <X11/Xatom.h>
#include <X11/Xlib.h>
#include <X11/Xos.h>
#include <X11/Xproto.h>

Atom XA_SCREENSAVER, XA_SCREENSAVER_VERSION, XA_SCREENSAVER_STATUS, XA_BLANK, XA_LOCK, XA_DEACTIVATE;


static XErrorHandler old_handler = 0;
static int IgnoreBadWindow( Display *dpy, XErrorEvent *error )
{
	ASSERT( old_handler );

	if( error->error_code == BadWindow )
		return 0;

	return (*old_handler)( dpy, error );
}

static bool HasWindowProperty( Display *dpy, Window window, Atom property )
{
	XSync( dpy, False );

	ASSERT( !old_handler );
	old_handler = XSetErrorHandler( IgnoreBadWindow );

	int num_prop;
	Atom *ret = XListProperties( dpy, window, &num_prop );

	bool bFound = false;
	if( ret )
	{
		for( int i = 0; !bFound && i < num_prop; ++i )
			if( ret[i] == property )
				bFound = true;
		XFree( ret );
	}

	XSetErrorHandler( old_handler );
	old_handler = 0;

	return bFound;
}

static Window FindXScreensaver( Display *dpy )
{
	const Window root = RootWindowOfScreen( DefaultScreenOfDisplay(dpy) );
	ASSERT( root );
	
	Window root2, parent, *kids;
	unsigned int nkids;
	ASSERT( XQueryTree(dpy, root, &root2, &parent, &kids, &nkids) );
	ASSERT( root == root2 && !parent );

	if( !kids || !nkids )
		return 0;

	Window ret = 0;
	for( unsigned i = 0; ret == 0 && i < nkids; ++i )
		if( HasWindowProperty( dpy, kids[i], XA_SCREENSAVER_VERSION ) )
			ret = kids[i];

	XFree( kids );
	return ret;
}

/* Note that if you kill xscreensaver and restart it, the status won't be updated
 * immediately.  (xscreensaver-command -time will still show "blanked" if you
 * kill it while blanked and restart it.) */
enum ScreenSaverStatus { SCREENSAVER_NOT_SHOWN, SCREENSAVER_SHOWN, SCREENSAVER_SHOWN_AND_LOCKED };
static ScreenSaverStatus GetScreenSaverStatus( Display *dpy )
{
	Atom type;
	int format;
	unsigned long nitems, bytesafter;
	Atom *data = 0;

	if( XGetWindowProperty( dpy, RootWindow(dpy, 0), XA_SCREENSAVER_STATUS, 0, 999, False, XA_INTEGER,
				&type, &format, &nitems, &bytesafter, (unsigned char **) &data ) != Success || type != XA_INTEGER || !data || nitems < 3 )
	{
		if( data )
			XFree( data );

		return SCREENSAVER_NOT_SHOWN;
	}
	
        const Atom status = data[0];
	XFree( data );

	if( status == XA_BLANK )
		return SCREENSAVER_SHOWN;
	if( status == XA_LOCK )
		return SCREENSAVER_SHOWN_AND_LOCKED;
	return SCREENSAVER_NOT_SHOWN;
}

void delay_xscreensaver( Display *dpy )
{
	ASSERT( dpy );

	XA_SCREENSAVER = XInternAtom( dpy, "SCREENSAVER", False );
	XA_SCREENSAVER_VERSION = XInternAtom( dpy, "_SCREENSAVER_VERSION",False );
	XA_SCREENSAVER_STATUS = XInternAtom( dpy, "_SCREENSAVER_STATUS", False );
	XA_DEACTIVATE = XInternAtom( dpy, "DEACTIVATE", False );
	XA_BLANK = XInternAtom( dpy, "BLANK", False );
	XA_LOCK = XInternAtom( dpy, "LOCK", False );

	XSync( dpy, 0 );

	if( GetScreenSaverStatus( dpy ) == SCREENSAVER_SHOWN_AND_LOCKED )
		return;

	Window window = FindXScreensaver( dpy );
	if( !window )
		return;

	XEvent event;
	event.xany.type = ClientMessage;
	event.xclient.display = dpy;
	event.xclient.window = window;
	event.xclient.message_type = XA_SCREENSAVER;
	event.xclient.format = 32;
	memset (&event.xclient.data, 0, sizeof(event.xclient.data));
	event.xclient.data.l[0] = (long) XA_DEACTIVATE;
	event.xclient.data.l[1] = 0;
	event.xclient.data.l[2] = 0;
	XSendEvent( dpy, window, False, 0L, &event );

	XSync (dpy, 0);
}

